# Copyright (C) 2022 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Starts STK Runtime or attaches to an already running STK Runtime, and provides access to the Object Model root."""

__all__ = ["STKRuntime", "STKRuntimeApplication"]

import atexit
import os
import pathlib
import socket

# The subprocess module is needed to start the backend.
# Excluding low severity bandit warning as the validity of the inputs is enforced.
import subprocess  # nosec B404

from .internal.apiutil import InterfaceProxy, read_registry_key, winreg_stk_binary_dir
from .internal.grpcutil import GrpcClient
from .stkobjects import STKObjectModelContext, STKObjectRoot
from .stkx import STKXApplication
from .utilities.exceptions import STKInitializationError
from .utilities.grpcutilities import (
    GrpcAuthenticationMode,
    GrpcCallBatcher,
    _get_authentication_mode_string,
    _validate_authentication_mode,
)


class STKRuntimeApplication(STKXApplication):
    """
    Interact with STK Runtime.

    Use STKRuntime.StartApplication() or STKRuntime.AttachToApplication()
    to obtain an initialized STKRuntimeApplication object.
    """

    def __init__(self):
        """Construct an object of type STKRuntimeApplication."""
        self.__dict__["_intf"] = InterfaceProxy()
        STKXApplication.__init__(self)
        self.__dict__["_root"] = None
        self.__dict__["_shutdown"] = False

    def _private_init(self, intf: InterfaceProxy):
        STKXApplication._private_init(self, intf)

    def __del__(self):
        """Destruct the STKRuntimeApplication object when all references to the object are deleted."""
        if self._intf:
            client: GrpcClient = self._intf.client
            client.terminate_connection(call_shutdown=self._shutdown)

    def new_object_root(self) -> STKObjectRoot:
        """May be used to obtain an Object Model Root from a running STK Engine application."""
        if self._intf:
            client: GrpcClient = self._intf.client
            root_unk = client.new_object_root()
            root = STKObjectRoot()
            root._private_init(root_unk)
            return root
        raise STKInitializationError("Not connected to the gRPC server.")

    def new_object_model_context(self) -> STKObjectModelContext:
        """May be used to obtain an Object Model Context from a running STK Engine application."""
        if self._intf:
            client: GrpcClient = self._intf.client
            context_unk = client.new_object_model_context()
            context = STKObjectModelContext()
            context._private_init(context_unk)
            return context
        raise STKInitializationError("Not connected to the gRPC server.")

    def set_grpc_options(self, options:dict) -> None:
        """
        Set advanced-usage options for the gRPC client.

        Available options include:
        { "collection iteration batch size" : int }. Number of items to preload while iterating
        through a collection object. Default is 100. Use 0 to indicate no limit (load entire collection).
        { "disable batching" : bool }. Disable all batching operations.
        { "release batch size" : int }. Number of interfaces to be garbage collected before
        sending the entire batch to STK to be released. Default value is 12.
        """
        if self._intf:
            client: GrpcClient = self._intf.client
            client.set_grpc_options(options)

    def new_grpc_call_batcher(self, max_batch:int=None, disable_batching:bool=False) -> GrpcCallBatcher:
        """
        Construct a GrpcCallBatcher linked to this gRPC client that may be used to improve API performance.

        max_batch is the maximum number of calls to batch together.
        Set disable_batching=True to disable batching operations for this batcher.
        See grpcutilities module for more information.
        """
        batcher = GrpcCallBatcher(disable_batching)
        batcher._private_init(self._intf.client, max_batch)
        return batcher

    def shutdown(self) -> None:
        """Shut down the STKRuntime application."""
        self.__dict__["_shutdown"] = True
        self._disconnect()

    def _disconnect(self) -> None:
        """Safely disconnect from STKRuntime."""
        if self._intf:
            client: GrpcClient = self._intf.client
            client.terminate_connection(call_shutdown=self._shutdown)
            self.__dict__["_intf"] = InterfaceProxy()

class STKRuntime(object):
    """Connect to STKRuntime using gRPC."""

    @staticmethod
    def start_application(grpc_host:str="127.0.0.1",
                         grpc_port:int=40704,
                         grpc_timeout_sec:int=60,
                         grpc_max_message_size:int=0,
                         user_control:bool=False,
                         no_graphics:bool=True,
                         grpc_allow_remote_host:bool=False,
                         grpc_server_cert:str=None,
                         grpc_server_key:str=None,
                         grpc_client_cert:str=None,
                         grpc_client_key:str=None,
                         grpc_ca:str=None,
                         grpc_uds_directory:str=None,
                         grpc_uds_id:str=None,
                         grpc_authentication_mode:GrpcAuthenticationMode=GrpcAuthenticationMode.DEFAULT) -> STKRuntimeApplication:
        """
        Create a new STK Runtime instance and attach to the remote host.

        grpc_host is the IP address or DNS name of the gRPC server.
        grpc_port is the integral port number that the gRPC server is using (valid values are integers from 0 to 65535).
        grpc_timeout_sec specifies the time allocated to wait for a grpc connection (seconds).
        grpc_max_message_size is the maximum size in bytes that the gRPC client can receive. Set to zero to use the gRPC default.
        user_control specifies if the application returns to the user's control
        (the application remains open) after terminating the Python API connection.
        no_graphics controls if runtime is started with or without graphics.
        Specify grpc_allow_remote_host = True to allow external connections, not allowed by default. Required when using 0.0.0.0 or
            other remote address as the grpc_host.
        grpc_server_cert is the path to the server certificate file. Required for mTLS authentication.
        grpc_server_key is the path to the server key file. Required for mTLS authentication.
        grpc_client_cert is the path to the client certificate file. Required for mTLS authentication.
        grpc_client_key is the path to the client key file. Required for mTLS authentication.
        grpc_ca is the path to the issuing certificate authority. Required for mTLS authentication.
        grpc_uds_directory is an optional override of the path to the directory for UDS socket files. Only supported on Linux.
        grpc_uds_id is the optional ID for UDS socket file naming (stk-runtime-grpc<-id>.sock). Only supported on Linux.
        grpc_authentication_mode is the method of client-server authentication to use for gRPC. Default is SINGLE_USER on Windows,
            UNIX_DOMAIN_SOCKET on Linux.
        """
        if grpc_port < 0 or grpc_port > 65535:
            raise STKInitializationError(f"{grpc_port} is not a valid port number for the gRPC server.")
        if grpc_host not in ["localhost", "127.0.0.1"]:
            if not grpc_allow_remote_host:
                raise RuntimeError("Remote host connections are not allowed. Use grpc_allow_remote_host to enable.")
            try:
                socket.inet_pton(socket.AF_INET, grpc_host)
            except OSError:
                try:
                    socket.inet_pton(socket.AF_INET6, grpc_host)
                except OSError:
                    raise STKInitializationError(f"Could not resolve host \"{grpc_host}\" for the gRPC server.")

        _validate_authentication_mode(grpc_authentication_mode, grpc_host)

        cmd_line = []
        if os.name != "nt":
            ld_env = os.getenv('LD_LIBRARY_PATH')
            if ld_env:
                for path in ld_env.split(':'):
                    stkruntime_path = (pathlib.Path(path) / "stkruntime").resolve()
                    if stkruntime_path.exists():
                        cmd_line = [stkruntime_path]
                        if any([grpc_uds_directory, grpc_uds_id]):
                            if grpc_uds_directory:
                                cmd_line.extend(["--grpcUdsDir", grpc_uds_directory])
                            if grpc_uds_id:
                                cmd_line.extend(["--grpcUdsId", grpc_uds_id])
                        else:
                            cmd_line.extend(["--grpcHost", grpc_host, "--grpcPort", str(grpc_port)])
                        break
            else:
                raise STKInitializationError("LD_LIBRARY_PATH not defined. Add STK bin directory to LD_LIBRARY_PATH before running.")
        else:
            clsid_stkxapplication = "{062AB565-B121-45B5-A9A9-B412CEFAB6A9}"
            stkx_dll_registry_value = read_registry_key(f"CLSID\\{clsid_stkxapplication}\\InprocServer32", silent_exception=True)
            stkruntime_path = None if stkx_dll_registry_value is None else pathlib.Path(stkx_dll_registry_value).parent / "STKRuntime.exe"
            if stkruntime_path is None or not stkruntime_path.exists():
                stkruntime_path = pathlib.Path(winreg_stk_binary_dir()) / "STKRuntime.exe"
                if not stkruntime_path.exists():
                    raise STKInitializationError("Could not find STKRuntime.exe. Verify STK installation.")
            cmd_line = [str(stkruntime_path.resolve()), "/grpcHost", grpc_host, "/grpcPort", str(grpc_port)]

        flag = '--' if os.name != 'nt' else '/'
        cmd_line.extend([f"{flag}grpcAuthMode", _get_authentication_mode_string(grpc_authentication_mode)])
        if grpc_server_cert:
            cmd_line.extend([f"{flag}grpcServerCert", str(pathlib.Path(grpc_server_cert).resolve())])
        if grpc_server_key:
            cmd_line.extend([f"{flag}grpcServerKey", str(pathlib.Path(grpc_server_key).resolve())])
        if grpc_ca:
            cmd_line.extend([f"{flag}grpcCa", str(pathlib.Path(grpc_ca).resolve())])
        if grpc_allow_remote_host:
            cmd_line.append(f"{flag}grpcAllowRemoteHost")
        if no_graphics:
            cmd_line.append(f"{flag}noGraphics")

        # Calling subprocess.Popen (without shell equals true) to start the backend.
        # Excluding low severity bandit check as the validity of the inputs has been ensured.
        p = subprocess.Popen(cmd_line) # nosec B603
        host = grpc_host
        # Ignoring B104 warning as it is a false positive. The hard-coded string "0.0.0.0" is being filtered
        # to ensure that it is not used.
        if grpc_host=="0.0.0.0": # nosec B104
            host = "127.0.0.1"
        try:
            app = STKRuntime.attach_to_application(host,
                                                 grpc_port,
                                                 grpc_timeout_sec,
                                                 grpc_max_message_size,
                                                 grpc_allow_remote_host,
                                                 grpc_client_cert,
                                                 grpc_client_key,
                                                 grpc_ca,
                                                 grpc_uds_directory,
                                                 grpc_uds_id,
                                                 grpc_authentication_mode)

            app.__dict__["_shutdown"] = not user_control
            return app
        except Exception:
            if not user_control:
                p.terminate()
            raise


    @staticmethod
    def attach_to_application(grpc_host:str="127.0.0.1",
                            grpc_port:int=40704,
                            grpc_timeout_sec:int=60,
                            grpc_max_message_size:int=0,
                            grpc_allow_remote_host:bool=False,
                            grpc_client_cert:str=None,
                            grpc_client_key:str=None,
                            grpc_ca:str=None,
                            grpc_uds_directory:str=None,
                            grpc_uds_id:str=None,
                            grpc_authentication_mode:GrpcAuthenticationMode=GrpcAuthenticationMode.DEFAULT) -> STKRuntimeApplication:
        """
        Attach to STKRuntime.

        grpc_host is the IP address or DNS name of the gRPC server.
        grpc_port is the integral port number that the gRPC server is using.
        grpc_timeout_sec specifies the time allocated to wait for a grpc connection (seconds).
        grpc_max_message_size is the maximum size in bytes that the gRPC client can receive. Set to zero to use the gRPC default.
        Specify grpc_allow_remote_host = True to allow external connections, not allowed by default. Required when using 0.0.0.0 or
            other remote address as the grpc_host.
        grpc_client_cert is the path to the client certificate file. Required for mTLS authentication.
        grpc_client_key is the path to the client key file. Required for mTLS authentication.
        grpc_ca is the path to the issuing certificate authority. Required for mTLS authentication.
        grpc_uds_directory is an optional override of the path to the directory for UDS socket files. Only supported on Linux.
        grpc_uds_id is the optional ID for UDS socket file naming (stk-runtime-grpc<-id>.sock). Only supported on Linux.
        grpc_authentication_mode is the method of client-server authentication to use for gRPC. Default is SINGLE_USER on Windows,
            UNIX_DOMAIN_SOCKET on Linux.
        """
        if grpc_host not in ["localhost", "127.0.0.1"] and not grpc_allow_remote_host:
            raise RuntimeError("Remote host connections are not allowed. Use grpc_allow_remote_host to enable.")

        _validate_authentication_mode(grpc_authentication_mode, grpc_host)

        if grpc_authentication_mode != GrpcAuthenticationMode.UNIX_DOMAIN_SOCKET and any([grpc_uds_directory, grpc_uds_id]):
            raise RuntimeError("grpc_authentication_mode must be set to UNIX_DOMAIN_SOCKET if UDS parameters are provided.")
        elif grpc_authentication_mode != GrpcAuthenticationMode.MUTUAL_TLS and any([grpc_client_cert, grpc_client_key, grpc_ca]):
            raise RuntimeError("grpc_authentication_mode must be set to MUTUAL_TLS if mutual TLS parameters are provided.")

        if grpc_authentication_mode == GrpcAuthenticationMode.UNIX_DOMAIN_SOCKET:
            if grpc_uds_directory:
                grpc_uds_directory = pathlib.Path(grpc_uds_directory).resolve()
            else:
                config_dir = os.getenv("STK_CONFIG_DIR")
                if config_dir:
                    grpc_uds_directory = (pathlib.Path(config_dir) / "STK12" / "Config" / ".conn").resolve()
                else:
                    raise RuntimeError("Please provide a value for grpc_uds_directory or set a valid STK_CONFIG_DIR " /
                                       "environment variable to use the default UDS directory.")

        client: GrpcClient = GrpcClient.new_client(grpc_host,
                                                   grpc_port,
                                                   grpc_timeout_sec,
                                                   grpc_max_message_size,
                                                   _get_authentication_mode_string(grpc_authentication_mode),
                                                   grpc_client_cert,
                                                   grpc_client_key,
                                                   grpc_ca,
                                                   grpc_uds_directory,
                                                   grpc_uds_id)

        if client is not None:
            app_intf = client.get_stk_application_interface()
            app = STKRuntimeApplication()
            app._private_init(app_intf)
            atexit.register(app._disconnect)
            return app
        raise STKInitializationError(f"Cannot connect to the gRPC server on {grpc_host}:{grpc_port}.")


# Copyright (C) 2022 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Information for a specific call captured while recording."""


class CallRecord:
    """Capture the information for a specific call."""

    def __init__(
        self,
        filename,
        type_name,
        member_name,
        lineno,
        end_lineno,
        col_offset,
        end_col_offset,
    ):
        """Construct a new CallRecord."""
        self.filename = filename
        self.type_name = type_name
        self.member_name = member_name
        self.lineno = lineno
        self.end_lineno = end_lineno
        self.col_offset = col_offset
        self.end_col_offset = end_col_offset

    def __eq__(self, other):
        """Compare this object with another object for equality."""
        if isinstance(other, CallRecord):
            return self.__dict__ == other.__dict__
        return NotImplemented

    def __hash__(self):
        """Return the hash value for the object."""
        return hash(
            (
                self.filename,
                self.type_name,
                self.member_name,
                self.lineno,
                self.end_lineno,
                self.col_offset,
                self.end_col_offset,
            )
        )

    def __lt__(self, other):
        """
        Define the behavior of the less-than operator (<) for this class.

        Sort by order of appearance in the source file.
        """
        if isinstance(other, CallRecord):
            return (
                self.filename,
                self.lineno,
                self.col_offset,
                self.end_lineno,
                self.end_col_offset,
            ).__lt__(
                (
                    other.filename,
                    other.lineno,
                    other.col_offset,
                    other.end_lineno,
                    other.end_col_offset,
                )
            )
        return NotImplemented



# Copyright (C) 2022 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""
PySTK Date Extension.

A set of utilities to facilitate the manipulation of dates and times in PySTK.
"""

from collections.abc import Callable

from matplotlib import units
from matplotlib.ticker import FuncFormatter
import numpy as np

from ansys.stk.core.stkobjects import STKObjectRoot
from ansys.stk.core.stkutil import Date, Quantity


class _STKDate:
    """Wrapper class associated with STK Date object."""

    def __init__(self, date: Date):
        """Create an STKDate object from Date."""
        self.stk_date: Date = date

    @classmethod
    def from_value_and_format(cls, root: STKObjectRoot, value: str, unit: str = "UTCG") -> "_STKDate":
        """Create a new STKDate object.

        Parameters
        ----------
        root : STKObjectRoot
            The STK object root.
        value : str
            String containing date to be parsed.
        unit : str
            String representing the unit the date is in (the default is UTCG).

        Returns
        -------
        ansys.stk.extensions.data_analysis.dates.STKDate
            The `STKDate` object.

        """
        return cls(root.conversion_utility.new_date(unit, value))

    def __sub__(self, date: "_STKDate") -> float:
        """Subtract an STKDate, returning the time difference in seconds."""
        span : Quantity =  self.stk_date.span(date.stk_date)
        if span.unit != "sec":
            span.convert_to_unit("sec")
        return span.value

    def __lt__(self, other: "_STKDate"):
        """Compare STKDate to STKDate."""
        if isinstance(other, _STKDate):
            return self - other < 0
        return NotImplemented

    def __gt__(self, other: "_STKDate"):
        """Compare STKDate to STKDate."""
        if isinstance(other, _STKDate):
            return self - other > 0
        return NotImplemented

    def __ge__(self, other: "_STKDate"):
        """Compare STKDate to STKDate."""
        if isinstance(other, _STKDate):
            return self > other or self == other
        return NotImplemented

    def __eq__(self, other: "_STKDate"):
        """Compare equality of STKDates."""
        if isinstance(other, _STKDate):
            return self.get_utcg() == other.get_utcg()
        return NotImplemented

    def __add__(self, seconds: float) -> Date:
        """Add seconds to the date."""
        return _STKDate(self.stk_date.add("sec", seconds))

    def add_duration(self, value: float, unit: str)-> "_STKDate":
        """Add the time duration in the given unit."""
        return _STKDate(self.stk_date.add(unit, value))

    def get_epsec(self) -> float:
        """Return the date in Epoch Seconds.

        Returns
        -------
        float
            The date in Epoch Seconds.

        """
        return float(self.stk_date.format('EpSec'))

    def get_utcg(self) -> str:
        """Return the date formatted in UTCG.

        Returns
        -------
        str
            The date formatted as UTCG.

        """
        return self.stk_date.format('UTCG')

    def format(self, unit: str) -> str:
        """Return the date formatted as any unit.

        Parameters
        ----------
        unit : str
            The unit to format the date as.

        Returns
        -------
        str
            The formatted date.

        """
        return self.stk_date.format(unit)

class _STKDateConverter(units.ConversionInterface):
        def __init__(self, root: STKObjectRoot, time_difference: float, unit_abbreviation: str, formatter: Callable[[float, float], str]):
            self.root = root
            self.time_difference = time_difference
            self.unit_abbreviation = unit_abbreviation
            self.formatter = formatter
            super().__init__()

        def default_units(self, x, axis):
            if isinstance(x, _STKDate):
                return "Date"
            else:
                return None

        def convert(self, value, unit, axis):
            """Convert an _STKDate object to a scalar."""
            if isinstance(value, _STKDate):
                return value.get_epsec()
            elif isinstance(value, (list, tuple, np.ndarray)) and all(isinstance(v, _STKDate) for v in value):
                 return [v.get_epsec() for v in value]
            else:
                return value

        def axisinfo(self, unit, axis):
            """Return major and minor tick locators and formatters."""
            def get_formatted_date_from_epsec(epsec : float):
                return (_STKDate.from_value_and_format(self.root, str(epsec), 'EpSec')).format(self.unit_abbreviation)

            def utcg_formatter(x : float, pos : float):
                utcg = get_formatted_date_from_epsec(x)
                # One second
                if  self.time_difference < 1:
                    return utcg.rsplit(' ', maxsplit=1)[-1]
                # One minute
                elif self.time_difference < 60:
                    return utcg.rsplit(' ', maxsplit=1)[-1][:-2]
                # Two days
                elif self.time_difference < 172800:
                    return utcg.rsplit(' ', maxsplit=1)[-1].rsplit(':', maxsplit=1)[0]
                # Two months
                elif self.time_difference < 5184000:
                    return utcg.rsplit(' ', maxsplit=2)[0]
                else:
                    return utcg.split(' ', maxsplit=1)[-1].rsplit(' ', maxsplit=1)[0]

            def default_formatter(x : float, pos : float):
                return get_formatted_date_from_epsec(x)

            if self.formatter:
                formatter = self.formatter
            elif self.unit_abbreviation == "UTCG":
                formatter = utcg_formatter
            else:
                formatter = default_formatter

            return units.AxisInfo(
                majfmt=FuncFormatter(formatter)
            )


# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from . import AgGrpcServices_pb2 as AgGrpcServices__pb2


class STKGrpcServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.GetConnectionMetadata = channel.unary_unary(
                '/AgGrpcServices.STKGrpcService/GetConnectionMetadata',
                request_serializer=AgGrpcServices__pb2.EmptyMessage.SerializeToString,
                response_deserializer=AgGrpcServices__pb2.ConnectionMetadata.FromString,
                )
        self.ShutDownServer = channel.unary_unary(
                '/AgGrpcServices.STKGrpcService/ShutDownServer',
                request_serializer=AgGrpcServices__pb2.ShutDownRequest.SerializeToString,
                response_deserializer=AgGrpcServices__pb2.EmptyMessage.FromString,
                )
        self.GetStkApplication = channel.unary_unary(
                '/AgGrpcServices.STKGrpcService/GetStkApplication',
                request_serializer=AgGrpcServices__pb2.EmptyMessage.SerializeToString,
                response_deserializer=AgGrpcServices__pb2.ObjReturn.FromString,
                )
        self.EngineNewRoot = channel.unary_unary(
                '/AgGrpcServices.STKGrpcService/EngineNewRoot',
                request_serializer=AgGrpcServices__pb2.EmptyMessage.SerializeToString,
                response_deserializer=AgGrpcServices__pb2.ObjReturn.FromString,
                )
        self.EngineNewRootContext = channel.unary_unary(
                '/AgGrpcServices.STKGrpcService/EngineNewRootContext',
                request_serializer=AgGrpcServices__pb2.EmptyMessage.SerializeToString,
                response_deserializer=AgGrpcServices__pb2.ObjReturn.FromString,
                )
        self.AddRef = channel.unary_unary(
                '/AgGrpcServices.STKGrpcService/AddRef',
                request_serializer=AgGrpcServices__pb2.RefCountRequest.SerializeToString,
                response_deserializer=AgGrpcServices__pb2.RefCountReturn.FromString,
                )
        self.Release = channel.unary_unary(
                '/AgGrpcServices.STKGrpcService/Release',
                request_serializer=AgGrpcServices__pb2.RefCountRequest.SerializeToString,
                response_deserializer=AgGrpcServices__pb2.RefCountReturn.FromString,
                )
        self.BatchedRelease = channel.unary_unary(
                '/AgGrpcServices.STKGrpcService/BatchedRelease',
                request_serializer=AgGrpcServices__pb2.BatchedReleaseRequest.SerializeToString,
                response_deserializer=AgGrpcServices__pb2.EmptyMessage.FromString,
                )
        self.SupportsInterface = channel.unary_unary(
                '/AgGrpcServices.STKGrpcService/SupportsInterface',
                request_serializer=AgGrpcServices__pb2.SupportsInterfaceRequest.SerializeToString,
                response_deserializer=AgGrpcServices__pb2.SupportsInterfaceReturn.FromString,
                )
        self.Invoke = channel.unary_unary(
                '/AgGrpcServices.STKGrpcService/Invoke',
                request_serializer=AgGrpcServices__pb2.InvokeRequest.SerializeToString,
                response_deserializer=AgGrpcServices__pb2.InvokeReturn.FromString,
                )
        self.GetProperty = channel.unary_unary(
                '/AgGrpcServices.STKGrpcService/GetProperty',
                request_serializer=AgGrpcServices__pb2.GetPropertyRequest.SerializeToString,
                response_deserializer=AgGrpcServices__pb2.GetPropertyReturn.FromString,
                )
        self.SetProperty = channel.unary_unary(
                '/AgGrpcServices.STKGrpcService/SetProperty',
                request_serializer=AgGrpcServices__pb2.SetPropertyRequest.SerializeToString,
                response_deserializer=AgGrpcServices__pb2.EmptyMessage.FromString,
                )
        self.EnumerateCollection = channel.unary_unary(
                '/AgGrpcServices.STKGrpcService/EnumerateCollection',
                request_serializer=AgGrpcServices__pb2.CollectionRequest.SerializeToString,
                response_deserializer=AgGrpcServices__pb2.CollectionReturn.FromString,
                )
        self.BatchedInvoke = channel.unary_unary(
                '/AgGrpcServices.STKGrpcService/BatchedInvoke',
                request_serializer=AgGrpcServices__pb2.BatchedInvokeRequest.SerializeToString,
                response_deserializer=AgGrpcServices__pb2.BatchedInvokeReturn.FromString,
                )
        self.StartEventLoop = channel.unary_stream(
                '/AgGrpcServices.STKGrpcService/StartEventLoop',
                request_serializer=AgGrpcServices__pb2.EmptyMessage.SerializeToString,
                response_deserializer=AgGrpcServices__pb2.EventLoopResponse.FromString,
                )
        self.StopEventLoop = channel.unary_unary(
                '/AgGrpcServices.STKGrpcService/StopEventLoop',
                request_serializer=AgGrpcServices__pb2.EventLoopData.SerializeToString,
                response_deserializer=AgGrpcServices__pb2.EmptyMessage.FromString,
                )
        self.AcknowledgeEvent = channel.unary_unary(
                '/AgGrpcServices.STKGrpcService/AcknowledgeEvent',
                request_serializer=AgGrpcServices__pb2.EventLoopData.SerializeToString,
                response_deserializer=AgGrpcServices__pb2.EmptyMessage.FromString,
                )
        self.Subscribe = channel.unary_unary(
                '/AgGrpcServices.STKGrpcService/Subscribe',
                request_serializer=AgGrpcServices__pb2.SubscriptionData.SerializeToString,
                response_deserializer=AgGrpcServices__pb2.EmptyMessage.FromString,
                )
        self.Unsubscribe = channel.unary_unary(
                '/AgGrpcServices.STKGrpcService/Unsubscribe',
                request_serializer=AgGrpcServices__pb2.SubscriptionData.SerializeToString,
                response_deserializer=AgGrpcServices__pb2.EmptyMessage.FromString,
                )


class STKGrpcServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def GetConnectionMetadata(self, request, context):
        """Server Operations
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ShutDownServer(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetStkApplication(self, request, context):
        """Object Model Entry Points
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def EngineNewRoot(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def EngineNewRootContext(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def AddRef(self, request, context):
        """Object/Interface Operations
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Release(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def BatchedRelease(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SupportsInterface(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Invoke(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetProperty(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetProperty(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def EnumerateCollection(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def BatchedInvoke(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def StartEventLoop(self, request, context):
        """Event Handling
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def StopEventLoop(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def AcknowledgeEvent(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Subscribe(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Unsubscribe(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_STKGrpcServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'GetConnectionMetadata': grpc.unary_unary_rpc_method_handler(
                    servicer.GetConnectionMetadata,
                    request_deserializer=AgGrpcServices__pb2.EmptyMessage.FromString,
                    response_serializer=AgGrpcServices__pb2.ConnectionMetadata.SerializeToString,
            ),
            'ShutDownServer': grpc.unary_unary_rpc_method_handler(
                    servicer.ShutDownServer,
                    request_deserializer=AgGrpcServices__pb2.ShutDownRequest.FromString,
                    response_serializer=AgGrpcServices__pb2.EmptyMessage.SerializeToString,
            ),
            'GetStkApplication': grpc.unary_unary_rpc_method_handler(
                    servicer.GetStkApplication,
                    request_deserializer=AgGrpcServices__pb2.EmptyMessage.FromString,
                    response_serializer=AgGrpcServices__pb2.ObjReturn.SerializeToString,
            ),
            'EngineNewRoot': grpc.unary_unary_rpc_method_handler(
                    servicer.EngineNewRoot,
                    request_deserializer=AgGrpcServices__pb2.EmptyMessage.FromString,
                    response_serializer=AgGrpcServices__pb2.ObjReturn.SerializeToString,
            ),
            'EngineNewRootContext': grpc.unary_unary_rpc_method_handler(
                    servicer.EngineNewRootContext,
                    request_deserializer=AgGrpcServices__pb2.EmptyMessage.FromString,
                    response_serializer=AgGrpcServices__pb2.ObjReturn.SerializeToString,
            ),
            'AddRef': grpc.unary_unary_rpc_method_handler(
                    servicer.AddRef,
                    request_deserializer=AgGrpcServices__pb2.RefCountRequest.FromString,
                    response_serializer=AgGrpcServices__pb2.RefCountReturn.SerializeToString,
            ),
            'Release': grpc.unary_unary_rpc_method_handler(
                    servicer.Release,
                    request_deserializer=AgGrpcServices__pb2.RefCountRequest.FromString,
                    response_serializer=AgGrpcServices__pb2.RefCountReturn.SerializeToString,
            ),
            'BatchedRelease': grpc.unary_unary_rpc_method_handler(
                    servicer.BatchedRelease,
                    request_deserializer=AgGrpcServices__pb2.BatchedReleaseRequest.FromString,
                    response_serializer=AgGrpcServices__pb2.EmptyMessage.SerializeToString,
            ),
            'SupportsInterface': grpc.unary_unary_rpc_method_handler(
                    servicer.SupportsInterface,
                    request_deserializer=AgGrpcServices__pb2.SupportsInterfaceRequest.FromString,
                    response_serializer=AgGrpcServices__pb2.SupportsInterfaceReturn.SerializeToString,
            ),
            'Invoke': grpc.unary_unary_rpc_method_handler(
                    servicer.Invoke,
                    request_deserializer=AgGrpcServices__pb2.InvokeRequest.FromString,
                    response_serializer=AgGrpcServices__pb2.InvokeReturn.SerializeToString,
            ),
            'GetProperty': grpc.unary_unary_rpc_method_handler(
                    servicer.GetProperty,
                    request_deserializer=AgGrpcServices__pb2.GetPropertyRequest.FromString,
                    response_serializer=AgGrpcServices__pb2.GetPropertyReturn.SerializeToString,
            ),
            'SetProperty': grpc.unary_unary_rpc_method_handler(
                    servicer.SetProperty,
                    request_deserializer=AgGrpcServices__pb2.SetPropertyRequest.FromString,
                    response_serializer=AgGrpcServices__pb2.EmptyMessage.SerializeToString,
            ),
            'EnumerateCollection': grpc.unary_unary_rpc_method_handler(
                    servicer.EnumerateCollection,
                    request_deserializer=AgGrpcServices__pb2.CollectionRequest.FromString,
                    response_serializer=AgGrpcServices__pb2.CollectionReturn.SerializeToString,
            ),
            'BatchedInvoke': grpc.unary_unary_rpc_method_handler(
                    servicer.BatchedInvoke,
                    request_deserializer=AgGrpcServices__pb2.BatchedInvokeRequest.FromString,
                    response_serializer=AgGrpcServices__pb2.BatchedInvokeReturn.SerializeToString,
            ),
            'StartEventLoop': grpc.unary_stream_rpc_method_handler(
                    servicer.StartEventLoop,
                    request_deserializer=AgGrpcServices__pb2.EmptyMessage.FromString,
                    response_serializer=AgGrpcServices__pb2.EventLoopResponse.SerializeToString,
            ),
            'StopEventLoop': grpc.unary_unary_rpc_method_handler(
                    servicer.StopEventLoop,
                    request_deserializer=AgGrpcServices__pb2.EventLoopData.FromString,
                    response_serializer=AgGrpcServices__pb2.EmptyMessage.SerializeToString,
            ),
            'AcknowledgeEvent': grpc.unary_unary_rpc_method_handler(
                    servicer.AcknowledgeEvent,
                    request_deserializer=AgGrpcServices__pb2.EventLoopData.FromString,
                    response_serializer=AgGrpcServices__pb2.EmptyMessage.SerializeToString,
            ),
            'Subscribe': grpc.unary_unary_rpc_method_handler(
                    servicer.Subscribe,
                    request_deserializer=AgGrpcServices__pb2.SubscriptionData.FromString,
                    response_serializer=AgGrpcServices__pb2.EmptyMessage.SerializeToString,
            ),
            'Unsubscribe': grpc.unary_unary_rpc_method_handler(
                    servicer.Unsubscribe,
                    request_deserializer=AgGrpcServices__pb2.SubscriptionData.FromString,
                    response_serializer=AgGrpcServices__pb2.EmptyMessage.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'AgGrpcServices.STKGrpcService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class STKGrpcService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def GetConnectionMetadata(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/AgGrpcServices.STKGrpcService/GetConnectionMetadata',
            AgGrpcServices__pb2.EmptyMessage.SerializeToString,
            AgGrpcServices__pb2.ConnectionMetadata.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ShutDownServer(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/AgGrpcServices.STKGrpcService/ShutDownServer',
            AgGrpcServices__pb2.ShutDownRequest.SerializeToString,
            AgGrpcServices__pb2.EmptyMessage.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetStkApplication(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/AgGrpcServices.STKGrpcService/GetStkApplication',
            AgGrpcServices__pb2.EmptyMessage.SerializeToString,
            AgGrpcServices__pb2.ObjReturn.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def EngineNewRoot(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/AgGrpcServices.STKGrpcService/EngineNewRoot',
            AgGrpcServices__pb2.EmptyMessage.SerializeToString,
            AgGrpcServices__pb2.ObjReturn.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def EngineNewRootContext(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/AgGrpcServices.STKGrpcService/EngineNewRootContext',
            AgGrpcServices__pb2.EmptyMessage.SerializeToString,
            AgGrpcServices__pb2.ObjReturn.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def AddRef(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/AgGrpcServices.STKGrpcService/AddRef',
            AgGrpcServices__pb2.RefCountRequest.SerializeToString,
            AgGrpcServices__pb2.RefCountReturn.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Release(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/AgGrpcServices.STKGrpcService/Release',
            AgGrpcServices__pb2.RefCountRequest.SerializeToString,
            AgGrpcServices__pb2.RefCountReturn.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def BatchedRelease(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/AgGrpcServices.STKGrpcService/BatchedRelease',
            AgGrpcServices__pb2.BatchedReleaseRequest.SerializeToString,
            AgGrpcServices__pb2.EmptyMessage.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SupportsInterface(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/AgGrpcServices.STKGrpcService/SupportsInterface',
            AgGrpcServices__pb2.SupportsInterfaceRequest.SerializeToString,
            AgGrpcServices__pb2.SupportsInterfaceReturn.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Invoke(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/AgGrpcServices.STKGrpcService/Invoke',
            AgGrpcServices__pb2.InvokeRequest.SerializeToString,
            AgGrpcServices__pb2.InvokeReturn.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetProperty(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/AgGrpcServices.STKGrpcService/GetProperty',
            AgGrpcServices__pb2.GetPropertyRequest.SerializeToString,
            AgGrpcServices__pb2.GetPropertyReturn.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetProperty(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/AgGrpcServices.STKGrpcService/SetProperty',
            AgGrpcServices__pb2.SetPropertyRequest.SerializeToString,
            AgGrpcServices__pb2.EmptyMessage.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def EnumerateCollection(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/AgGrpcServices.STKGrpcService/EnumerateCollection',
            AgGrpcServices__pb2.CollectionRequest.SerializeToString,
            AgGrpcServices__pb2.CollectionReturn.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def BatchedInvoke(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/AgGrpcServices.STKGrpcService/BatchedInvoke',
            AgGrpcServices__pb2.BatchedInvokeRequest.SerializeToString,
            AgGrpcServices__pb2.BatchedInvokeReturn.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def StartEventLoop(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/AgGrpcServices.STKGrpcService/StartEventLoop',
            AgGrpcServices__pb2.EmptyMessage.SerializeToString,
            AgGrpcServices__pb2.EventLoopResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def StopEventLoop(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/AgGrpcServices.STKGrpcService/StopEventLoop',
            AgGrpcServices__pb2.EventLoopData.SerializeToString,
            AgGrpcServices__pb2.EmptyMessage.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def AcknowledgeEvent(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/AgGrpcServices.STKGrpcService/AcknowledgeEvent',
            AgGrpcServices__pb2.EventLoopData.SerializeToString,
            AgGrpcServices__pb2.EmptyMessage.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Subscribe(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/AgGrpcServices.STKGrpcService/Subscribe',
            AgGrpcServices__pb2.SubscriptionData.SerializeToString,
            AgGrpcServices__pb2.EmptyMessage.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Unsubscribe(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/AgGrpcServices.STKGrpcService/Unsubscribe',
            AgGrpcServices__pb2.SubscriptionData.SerializeToString,
            AgGrpcServices__pb2.EmptyMessage.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



# Copyright (C) 2022 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Provides graphs for Access objects."""

import collections.abc
import typing

import matplotlib

from ansys.stk.core.stkobjects import Access
from ansys.stk.extensions.data_analysis.graphs._common_graphs import _access_duration_pie_chart
from ansys.stk.extensions.data_analysis.graphs.graph_helpers import (
    _get_access_data,
    interval_pie_chart,
    interval_plot,
    line_chart,
    polar_chart,
)


def access_duration_pie_chart(stk_object :Access, start_time: typing.Any = None, stop_time : typing.Any = None, colormap: matplotlib.colors.Colormap = None) -> tuple[matplotlib.figure.Figure, matplotlib.axes.Axes]:
    r"""Create a pie chart of the durations of the access intervals.

    This graph wrapper was generated from `AGI\\STK12\\STKData\\Styles\\Access\\Access Duration.rsg`.

    Parameters
    ----------
    stk_object : ansys.stk.core.stkobjects.Access
        The STK Access object.
    start_time : typing.Any
        The start time of the calculation (the default is None, which implies using the scenario start time).
    stop_time : typing.Any
        The stop time of the calculation (the default is None, which implies using the scenario stop time).
    colormap : matplotlib.colors.Colormap
        The colormap with which to color the data (the default is None).

    Returns
    -------
    matplotlib.figure.Figure
        The newly created figure.
    matplotlib.axes.Axes
        The newly created axes.
    """
    return _access_duration_pie_chart(stk_object, start_time, stop_time, colormap)

def access_interval_graph(stk_object :Access, start_time: typing.Any = None, stop_time : typing.Any = None, colormap: matplotlib.colors.Colormap = None,  time_unit_abbreviation: str = 'UTCG', formatter: collections.abc.Callable[[float, float], str] = None) -> tuple[matplotlib.figure.Figure, matplotlib.axes.Axes]:
    r"""Create an interval graph of the access intervals.

    This graph wrapper was generated from `AGI\\STK12\\STKData\\Styles\\Access\\Access.rsg`.

    Parameters
    ----------
    stk_object : ansys.stk.core.stkobjects.Access
        The STK Access object.
    start_time : typing.Any
        The start time of the calculation (the default is None, which implies using the scenario start time).
    stop_time : typing.Any
        The stop time of the calculation (the default is None, which implies using the scenario stop time).
    colormap : matplotlib.colors.Colormap
        The colormap with which to color the data (the default is None).
    time_unit_abbreviation : str
        The time unit for formatting (the default is "UTCG").
    formatter : collections.abc.Callable[[float, float], str]
        The formatter for time axes (the default is None).

    Returns
    -------
    matplotlib.figure.Figure
        The newly created figure.
    matplotlib.axes.Axes
        The newly created axes.
    """
    root = stk_object.base.root
    start_time = start_time or root.current_scenario.start_time
    stop_time = stop_time or root.current_scenario.stop_time
    df = stk_object.data_providers.item('Access Data').execute_elements(start_time, stop_time, ['Start Time', 'Stop Time']).data_sets.to_pandas_dataframe()
    elements=[(('start time', 'None'),('stop time', 'None'))]
    return interval_plot([df], root, elements, [], ['start time','stop time'], 'Time', 'Access Times', colormap=colormap,  time_unit_abbreviation= time_unit_abbreviation, formatter= formatter)

def aer_line_chart(stk_object :Access, start_time : typing.Any = None, stop_time :typing.Any = None, step : float = 60, colormap: matplotlib.colors.Colormap = None,  time_unit_abbreviation: str = 'UTCG', formatter: collections.abc.Callable[[float, float], str] = None) -> tuple[matplotlib.figure.Figure, matplotlib.axes.Axes]:
    r"""Create a plot of the azimuth, elevation, and range values for the relative position vector between the base object and the target object, during access intervals.

    The relative position includes the effects of light time delay and aberration as set by the computational settings of the access. Az-El values are computed with respect to the default AER frame of the selected object of the Access Tool.

    This graph wrapper was generated from `AGI\\STK12\\STKData\\Styles\\Access\\AER.rsg`.

    Parameters
    ----------
    stk_object : ansys.stk.core.stkobjects.Access
        The STK Access object.
    start_time : typing.Any
        The start time of the calculation (the default is None, which implies using the scenario start time).
    stop_time : typing.Any
        The stop time of the calculation (the default is None, which implies using the scenario stop time).
    step_time : float
        The step time for the calculation (the default is 60 seconds).
    colormap : matplotlib.colors.Colormap
        The colormap with which to color the data (the default is None).
    time_unit_abbreviation : str
        The time unit for formatting (the default is "UTCG").
    formatter : collections.abc.Callable[[float, float], str]
        The formatter for time axes (the default is None).

    Returns
    -------
    matplotlib.figure.Figure
        The newly created figure.
    matplotlib.axes.Axes
        The newly created axes.
    """
    root = stk_object.base.root
    start_time = start_time or root.current_scenario.start_time
    stop_time = stop_time or root.current_scenario.stop_time
    data = _get_access_data(stk_object, 'AER Data', True, 'Default', ['Time', 'Elevation', 'Azimuth', 'Range'], start_time, stop_time, step)
    axes = [{'use_unit' : True, 'unit_squared': None, 'ylog10': False, 'y2log10': False, 'label': 'Longitude/Angle', 'lines': [
            {'y_name':'azimuth', 'label':'Azimuth', 'use_unit':True, 'unit_squared': None, 'dimension': 'Longitude'},
            {'y_name':'elevation', 'label':'Elevation', 'use_unit':True, 'unit_squared': None, 'dimension': 'Angle'}]},
            {'use_unit' : True, 'unit_squared': None, 'ylog10': False, 'y2log10': False, 'label': 'Distance', 'lines': [
            {'y_name':'range', 'label':'Range', 'use_unit':True, 'unit_squared': None, 'dimension': 'Distance'}]}]
    return line_chart(data, root, ['azimuth','elevation','range'], ['time'], axes, 'time', 'Time', 'AER', colormap=colormap,  time_unit_abbreviation= time_unit_abbreviation, formatter= formatter)

def angular_rates_line_chart(stk_object :Access, start_time : typing.Any = None, stop_time :typing.Any = None, step : float = 60, colormap: matplotlib.colors.Colormap = None,  time_unit_abbreviation: str = 'UTCG', formatter: collections.abc.Callable[[float, float], str] = None) -> tuple[matplotlib.figure.Figure, matplotlib.axes.Axes]:
    r"""Create a plot of the azimuth rate, elevation rate, and angular rate over time, during each access interval, from the perspective of the selected object in the Access Tool.

    The azimuth rate, elevation rate, and angular rate are available only if the selected object supports that metric as an access constraint: the value being reported is that as computed by that access constraint.

    This graph wrapper was generated from `AGI\\STK12\\STKData\\Styles\\Access\\Angular Rates.rsg`.

    Parameters
    ----------
    stk_object : ansys.stk.core.stkobjects.Access
        The STK Access object.
    start_time : typing.Any
        The start time of the calculation (the default is None, which implies using the scenario start time).
    stop_time : typing.Any
        The stop time of the calculation (the default is None, which implies using the scenario stop time).
    step_time : float
        The step time for the calculation (the default is 60 seconds).
    colormap : matplotlib.colors.Colormap
        The colormap with which to color the data (the default is None).
    time_unit_abbreviation : str
        The time unit for formatting (the default is "UTCG").
    formatter : collections.abc.Callable[[float, float], str]
        The formatter for time axes (the default is None).

    Returns
    -------
    matplotlib.figure.Figure
        The newly created figure.
    matplotlib.axes.Axes
        The newly created axes.
    """
    root = stk_object.base.root
    start_time = start_time or root.current_scenario.start_time
    stop_time = stop_time or root.current_scenario.stop_time
    data = _get_access_data(stk_object, 'Constraint Data', False, None, ['FromElevationRate', 'Time', 'FromAngularRate', 'FromAzimuthRate'], start_time, stop_time, step)
    axes = [{'use_unit' : True, 'unit_squared': None, 'ylog10': False, 'y2log10': False, 'label': 'Angle Rate', 'lines': [
            {'y_name':'fromangularrate', 'label':'FromAngularRate', 'use_unit':True, 'unit_squared': None, 'dimension': 'AngleRate'},
            {'y_name':'fromazimuthrate', 'label':'FromAzimuthRate', 'use_unit':True, 'unit_squared': None, 'dimension': 'AngleRate'},
            {'y_name':'fromelevationrate', 'label':'FromElevationRate', 'use_unit':True, 'unit_squared': None, 'dimension': 'AngleRate'}]}]
    return line_chart(data, root, ['fromangularrate','fromazimuthrate','fromelevationrate'], ['time'], axes, 'time', 'Time', 'Angular Rates', colormap=colormap,  time_unit_abbreviation= time_unit_abbreviation, formatter= formatter)

def az_el_polar_center_90_graph(stk_object :Access, start_time : typing.Any = None, stop_time :typing.Any = None, step : float = 60, colormap: matplotlib.colors.Colormap = None) -> tuple[matplotlib.figure.Figure, matplotlib.axes.Axes]:
    r"""Create a polar plot with elevation as radius and azimuth as angle theta over time, during access intervals.

    The azimuth and elevation describe the relative position vector between the base object and the target object. The relative position includes the effects of light time delay and aberration as set by the computational settings of the access. Az-El values are computed with respect to the default AER frame of the selected object of the Access Tool.

    This graph wrapper was generated from `AGI\\STK12\\STKData\\Styles\\Access\\Az El Polar.rsg`.

    Parameters
    ----------
    stk_object : ansys.stk.core.stkobjects.Access
        The STK Access object.
    start_time : typing.Any
        The start time of the calculation (the default is None, which implies using the scenario start time).
    stop_time : typing.Any
        The stop time of the calculation (the default is None, which implies using the scenario stop time).
    step_time : float
        The step time for the calculation (the default is 60 seconds).
    colormap : matplotlib.colors.Colormap
        The colormap with which to color the data (the default is None).

    Returns
    -------
    matplotlib.figure.Figure
        The newly created figure.
    matplotlib.axes.Axes
        The newly created axes.
    """
    root = stk_object.base.root
    start_time = start_time or root.current_scenario.start_time
    stop_time = stop_time or root.current_scenario.stop_time
    data = _get_access_data(stk_object, 'AER Data', True, 0, ['Elevation', 'Azimuth'], start_time, stop_time, step)
    axis={'use_unit' : True, 'unit_squared': False, 'label': 'Angle', 'lines': [
        {'y_name':'elevation','x_name':'azimuth', 'label':'Azimuth', 'use_unit':True, 'unit_squared': False, 'dimension': 'Angle'}
        ]}
    return polar_chart(data, root, ['elevation','azimuth'], axis, 'Az El Polar', convert_negative_r = False, origin_0 = True, colormap=colormap)

def bit_error_rate_line_chart(stk_object :Access, start_time : typing.Any = None, stop_time :typing.Any = None, step : float = 60, colormap: matplotlib.colors.Colormap = None,  time_unit_abbreviation: str = 'UTCG', formatter: collections.abc.Callable[[float, float], str] = None) -> tuple[matplotlib.figure.Figure, matplotlib.axes.Axes]:
    r"""Plot the bit error rate (BER) over time, during each access interval.

    This graph wrapper was generated from `AGI\\STK12\\STKData\\Styles\\Access\\Bit_Error_Rate.rsg`.

    Parameters
    ----------
    stk_object : ansys.stk.core.stkobjects.Access
        The STK Access object.
    start_time : typing.Any
        The start time of the calculation (the default is None, which implies using the scenario start time).
    stop_time : typing.Any
        The stop time of the calculation (the default is None, which implies using the scenario stop time).
    step_time : float
        The step time for the calculation (the default is 60 seconds).
    colormap : matplotlib.colors.Colormap
        The colormap with which to color the data (the default is None).
    time_unit_abbreviation : str
        The time unit for formatting (the default is "UTCG").
    formatter : collections.abc.Callable[[float, float], str]
        The formatter for time axes (the default is None).

    Returns
    -------
    matplotlib.figure.Figure
        The newly created figure.
    matplotlib.axes.Axes
        The newly created axes.
    """
    root = stk_object.base.root
    start_time = start_time or root.current_scenario.start_time
    stop_time = stop_time or root.current_scenario.stop_time
    data = _get_access_data(stk_object, 'Link Information', False, None, ['Time', 'BER'], start_time, stop_time, step)
    axes = [{'use_unit' : False, 'unit_squared': None, 'ylog10': False, 'y2log10': False, 'label': 'Value', 'lines': [
            {'y_name':'ber', 'label':'BER', 'use_unit':False, 'unit_squared': None, 'dimension': 'None'}]}]
    return line_chart(data, root, ['ber'], ['time'], axes, 'time', 'Time', 'Bit Error Rate', colormap=colormap,  time_unit_abbreviation= time_unit_abbreviation, formatter= formatter)

def carrier_to_noise_ratio_line_chart(stk_object :Access, start_time : typing.Any = None, stop_time :typing.Any = None, step : float = 60, colormap: matplotlib.colors.Colormap = None,  time_unit_abbreviation: str = 'UTCG', formatter: collections.abc.Callable[[float, float], str] = None) -> tuple[matplotlib.figure.Figure, matplotlib.axes.Axes]:
    r"""Plot the carrier to noise ratio (C/N) over time, during each access interval.

    This graph wrapper was generated from `AGI\\STK12\\STKData\\Styles\\Access\\Carrier_to_Noise_Ratio.rsg`.

    Parameters
    ----------
    stk_object : ansys.stk.core.stkobjects.Access
        The STK Access object.
    start_time : typing.Any
        The start time of the calculation (the default is None, which implies using the scenario start time).
    stop_time : typing.Any
        The stop time of the calculation (the default is None, which implies using the scenario stop time).
    step_time : float
        The step time for the calculation (the default is 60 seconds).
    colormap : matplotlib.colors.Colormap
        The colormap with which to color the data (the default is None).
    time_unit_abbreviation : str
        The time unit for formatting (the default is "UTCG").
    formatter : collections.abc.Callable[[float, float], str]
        The formatter for time axes (the default is None).

    Returns
    -------
    matplotlib.figure.Figure
        The newly created figure.
    matplotlib.axes.Axes
        The newly created axes.
    """
    root = stk_object.base.root
    start_time = start_time or root.current_scenario.start_time
    stop_time = stop_time or root.current_scenario.stop_time
    data = _get_access_data(stk_object, 'Link Information', False, None, ['Time', 'C/N'], start_time, stop_time, step)
    axes = [{'use_unit' : True, 'unit_squared': None, 'ylog10': False, 'y2log10': False, 'label': 'Ratio', 'lines': [
            {'y_name':'c/n', 'label':'C/N', 'use_unit':True, 'unit_squared': None, 'dimension': 'Ratio'}]}]
    return line_chart(data, root, ['c/n'], ['time'], axes, 'time', 'Time', 'Carrier to Noise Ratio', colormap=colormap,  time_unit_abbreviation= time_unit_abbreviation, formatter= formatter)

def cumulative_dwell_cumulative_pie_chart(stk_object :Access, start_time: typing.Any = None, stop_time : typing.Any = None, color_list: list[typing.Any] = None) -> tuple[matplotlib.figure.Figure, matplotlib.axes.Axes]:
    r"""Graph access interval durations as a cumulative pie chart.

    This graph wrapper was generated from `AGI\\STK12\\STKData\\Styles\\Access\\Cumulative Dwell.rsg`.

    Parameters
    ----------
    stk_object : ansys.stk.core.stkobjects.Access
        The STK Access object.
    start_time : typing.Any
        The start time of the calculation (the default is None, which implies using the scenario start time).
    stop_time : typing.Any
        The stop time of the calculation (the default is None, which implies using the scenario stop time).
    color_list : list of typing.Any
        The colors with which to color the pie chart slices (the default is None). Must have length >= 2.

    Returns
    -------
    matplotlib.figure.Figure
        The newly created figure.
    matplotlib.axes.Axes
        The newly created axes.
    """
    root = stk_object.base.root
    start_time = start_time or root.current_scenario.start_time
    stop_time = stop_time or root.current_scenario.stop_time
    df = stk_object.data_providers.item('Access Data').execute_elements(start_time, stop_time, ['Access Number', 'Stop Time', 'Start Time', 'Duration']).data_sets.to_pandas_dataframe()
    return interval_pie_chart(root, df, ['duration'], ['start time','stop time'], 'start time', 'stop time', start_time, stop_time, 'Cumulative Dwell', 'Time', True, color_list = color_list)

def ebno_line_chart(stk_object :Access, start_time : typing.Any = None, stop_time :typing.Any = None, step : float = 60, colormap: matplotlib.colors.Colormap = None,  time_unit_abbreviation: str = 'UTCG', formatter: collections.abc.Callable[[float, float], str] = None) -> tuple[matplotlib.figure.Figure, matplotlib.axes.Axes]:
    r"""Plot the energy per bit to noise ratio (Eb/No) over time, during each access interval.

    This graph wrapper was generated from `AGI\\STK12\\STKData\\Styles\\Access\\EbNo.rsg`.

    Parameters
    ----------
    stk_object : ansys.stk.core.stkobjects.Access
        The STK Access object.
    start_time : typing.Any
        The start time of the calculation (the default is None, which implies using the scenario start time).
    stop_time : typing.Any
        The stop time of the calculation (the default is None, which implies using the scenario stop time).
    step_time : float
        The step time for the calculation (the default is 60 seconds).
    colormap : matplotlib.colors.Colormap
        The colormap with which to color the data (the default is None).
    time_unit_abbreviation : str
        The time unit for formatting (the default is "UTCG").
    formatter : collections.abc.Callable[[float, float], str]
        The formatter for time axes (the default is None).

    Returns
    -------
    matplotlib.figure.Figure
        The newly created figure.
    matplotlib.axes.Axes
        The newly created axes.
    """
    root = stk_object.base.root
    start_time = start_time or root.current_scenario.start_time
    stop_time = stop_time or root.current_scenario.stop_time
    data = _get_access_data(stk_object, 'Link Information', False, None, ['Eb/No', 'Time'], start_time, stop_time, step)
    axes = [{'use_unit' : True, 'unit_squared': None, 'ylog10': False, 'y2log10': False, 'label': 'Ratio', 'lines': [
            {'y_name':'eb/no', 'label':'Eb/No', 'use_unit':True, 'unit_squared': None, 'dimension': 'Ratio'}]}]
    return line_chart(data, root, ['eb/no'], ['time'], axes, 'time', 'Time', 'EbNo', colormap=colormap,  time_unit_abbreviation= time_unit_abbreviation, formatter= formatter)

def elevation_angle_line_chart(stk_object :Access, start_time : typing.Any = None, stop_time :typing.Any = None, step : float = 60, colormap: matplotlib.colors.Colormap = None,  time_unit_abbreviation: str = 'UTCG', formatter: collections.abc.Callable[[float, float], str] = None) -> tuple[matplotlib.figure.Figure, matplotlib.axes.Axes]:
    r"""Create a plot of the elevation angle and its rate over time, during each access interval, from the perspective of the selected object in the Access Tool.

    The elevation angle value is that as computed by the elevation constraint for the selected object. The elevation rate is available only if the selected object supports that metric as an access constraint: the value being reported is that as computed by that access constraint.

    This graph wrapper was generated from `AGI\\STK12\\STKData\\Styles\\Access\\Elevation Angle.rsg`.

    Parameters
    ----------
    stk_object : ansys.stk.core.stkobjects.Access
        The STK Access object.
    start_time : typing.Any
        The start time of the calculation (the default is None, which implies using the scenario start time).
    stop_time : typing.Any
        The stop time of the calculation (the default is None, which implies using the scenario stop time).
    step_time : float
        The step time for the calculation (the default is 60 seconds).
    colormap : matplotlib.colors.Colormap
        The colormap with which to color the data (the default is None).
    time_unit_abbreviation : str
        The time unit for formatting (the default is "UTCG").
    formatter : collections.abc.Callable[[float, float], str]
        The formatter for time axes (the default is None).

    Returns
    -------
    matplotlib.figure.Figure
        The newly created figure.
    matplotlib.axes.Axes
        The newly created axes.
    """
    root = stk_object.base.root
    start_time = start_time or root.current_scenario.start_time
    stop_time = stop_time or root.current_scenario.stop_time
    data = _get_access_data(stk_object, 'Constraint Data', False, None, ['FromElevationRate', 'Time', 'FromElevationAngle'], start_time, stop_time, step)
    axes = [{'use_unit' : True, 'unit_squared': None, 'ylog10': False, 'y2log10': False, 'label': 'Angle', 'lines': [
            {'y_name':'fromelevationangle', 'label':'FromElevationAngle', 'use_unit':True, 'unit_squared': None, 'dimension': 'Angle'}]},
            {'use_unit' : True, 'unit_squared': None, 'ylog10': False, 'y2log10': False, 'label': 'Angle Rate', 'lines': [
            {'y_name':'fromelevationrate', 'label':'FromElevationRate', 'use_unit':True, 'unit_squared': None, 'dimension': 'AngleRate'}]}]
    return line_chart(data, root, ['fromelevationangle','fromelevationrate'], ['time'], axes, 'time', 'Time', 'Elevation', colormap=colormap,  time_unit_abbreviation= time_unit_abbreviation, formatter= formatter)

def gaps_interval_graph(stk_object :Access, start_time: typing.Any = None, stop_time : typing.Any = None, colormap: matplotlib.colors.Colormap = None,  time_unit_abbreviation: str = 'UTCG', formatter: collections.abc.Callable[[float, float], str] = None) -> tuple[matplotlib.figure.Figure, matplotlib.axes.Axes]:
    r"""Create an interval graph of the intervals where access does not exist between the objects.

    This graph wrapper was generated from `AGI\\STK12\\STKData\\Styles\\Access\\Gaps.rsg`.

    Parameters
    ----------
    stk_object : ansys.stk.core.stkobjects.Access
        The STK Access object.
    start_time : typing.Any
        The start time of the calculation (the default is None, which implies using the scenario start time).
    stop_time : typing.Any
        The stop time of the calculation (the default is None, which implies using the scenario stop time).
    colormap : matplotlib.colors.Colormap
        The colormap with which to color the data (the default is None).
    time_unit_abbreviation : str
        The time unit for formatting (the default is "UTCG").
    formatter : collections.abc.Callable[[float, float], str]
        The formatter for time axes (the default is None).

    Returns
    -------
    matplotlib.figure.Figure
        The newly created figure.
    matplotlib.axes.Axes
        The newly created axes.
    """
    root = stk_object.base.root
    start_time = start_time or root.current_scenario.start_time
    stop_time = stop_time or root.current_scenario.stop_time
    df = stk_object.data_providers.item('Access Gaps').execute_elements(start_time, stop_time, ['Start Time', 'Stop Time']).data_sets.to_pandas_dataframe()
    elements=[(('start time', 'None'),('stop time', 'None'))]
    return interval_plot([df], root, elements, [], ['start time','stop time'], 'Time', 'Access Gap Periods', colormap=colormap,  time_unit_abbreviation= time_unit_abbreviation, formatter= formatter)

def probability_of_detection_line_chart(stk_object :Access, start_time : typing.Any = None, stop_time :typing.Any = None, step : float = 60, colormap: matplotlib.colors.Colormap = None,  time_unit_abbreviation: str = 'UTCG', formatter: collections.abc.Callable[[float, float], str] = None) -> tuple[matplotlib.figure.Figure, matplotlib.axes.Axes]:
    r"""Graph the probability of a radar pulse search detection versus time.

    This graph wrapper was generated from `AGI\\STK12\\STKData\\Styles\\Access\\Probability_of_Detection.rsg`.

    Parameters
    ----------
    stk_object : ansys.stk.core.stkobjects.Access
        The STK Access object.
    start_time : typing.Any
        The start time of the calculation (the default is None, which implies using the scenario start time).
    stop_time : typing.Any
        The stop time of the calculation (the default is None, which implies using the scenario stop time).
    step_time : float
        The step time for the calculation (the default is 60 seconds).
    colormap : matplotlib.colors.Colormap
        The colormap with which to color the data (the default is None).
    time_unit_abbreviation : str
        The time unit for formatting (the default is "UTCG").
    formatter : collections.abc.Callable[[float, float], str]
        The formatter for time axes (the default is None).

    Returns
    -------
    matplotlib.figure.Figure
        The newly created figure.
    matplotlib.axes.Axes
        The newly created axes.
    """
    root = stk_object.base.root
    start_time = start_time or root.current_scenario.start_time
    stop_time = stop_time or root.current_scenario.stop_time
    data = _get_access_data(stk_object, 'Radar SearchTrack', False, None, ['Time', 'S/T PDet1'], start_time, stop_time, step)
    axes = [{'use_unit' : False, 'unit_squared': None, 'ylog10': False, 'y2log10': False, 'label': 'Value', 'lines': [
            {'y_name':'s/t pdet1', 'label':'S/T PDet1', 'use_unit':False, 'unit_squared': None, 'dimension': 'None'}]}]
    return line_chart(data, root, ['s/t pdet1'], ['time'], axes, 'time', 'Time', 'Probability of Detection', colormap=colormap,  time_unit_abbreviation= time_unit_abbreviation, formatter= formatter)

def radar_antenna_gain_line_chart(stk_object :Access, start_time : typing.Any = None, stop_time :typing.Any = None, step : float = 60, colormap: matplotlib.colors.Colormap = None,  time_unit_abbreviation: str = 'UTCG', formatter: collections.abc.Callable[[float, float], str] = None) -> tuple[matplotlib.figure.Figure, matplotlib.axes.Axes]:
    r"""Graph the antenna gain (value toward the Az, El direction)for both receiver and transmitter versus time.

    This graph wrapper was generated from `AGI\\STK12\\STKData\\Styles\\Access\\Radar Antenna Gain.rsg`.

    Parameters
    ----------
    stk_object : ansys.stk.core.stkobjects.Access
        The STK Access object.
    start_time : typing.Any
        The start time of the calculation (the default is None, which implies using the scenario start time).
    stop_time : typing.Any
        The stop time of the calculation (the default is None, which implies using the scenario stop time).
    step_time : float
        The step time for the calculation (the default is 60 seconds).
    colormap : matplotlib.colors.Colormap
        The colormap with which to color the data (the default is None).
    time_unit_abbreviation : str
        The time unit for formatting (the default is "UTCG").
    formatter : collections.abc.Callable[[float, float], str]
        The formatter for time axes (the default is None).

    Returns
    -------
    matplotlib.figure.Figure
        The newly created figure.
    matplotlib.axes.Axes
        The newly created axes.
    """
    root = stk_object.base.root
    start_time = start_time or root.current_scenario.start_time
    stop_time = stop_time or root.current_scenario.stop_time
    data = _get_access_data(stk_object, 'Radar Antenna', False, None, ['Time', 'Rcvr Ant Gain', 'Xmtr Ant Gain'], start_time, stop_time, step)
    axes = [{'use_unit' : True, 'unit_squared': None, 'ylog10': False, 'y2log10': False, 'label': 'Ratio', 'lines': [
            {'y_name':'rcvr ant gain', 'label':'Rcvr Ant Gain', 'use_unit':True, 'unit_squared': None, 'dimension': 'Ratio'}]},
            {'use_unit' : True, 'unit_squared': None, 'ylog10': False, 'y2log10': False, 'label': 'Ratio', 'lines': [
            {'y_name':'xmtr ant gain', 'label':'Xmtr Ant Gain', 'use_unit':True, 'unit_squared': None, 'dimension': 'Ratio'}]}]
    return line_chart(data, root, ['rcvr ant gain','xmtr ant gain'], ['time'], axes, 'time', 'Time', 'Radar Antenna Gain', colormap=colormap,  time_unit_abbreviation= time_unit_abbreviation, formatter= formatter)

def radar_propagation_loss_line_chart(stk_object :Access, start_time : typing.Any = None, stop_time :typing.Any = None, step : float = 60, colormap: matplotlib.colors.Colormap = None,  time_unit_abbreviation: str = 'UTCG', formatter: collections.abc.Callable[[float, float], str] = None) -> tuple[matplotlib.figure.Figure, matplotlib.axes.Axes]:
    r"""Graph the receive and transmit total propagation attenuation values for the primary polarization signal channel versus time.

    This graph wrapper was generated from `AGI\\STK12\\STKData\\Styles\\Access\\Radar Propagation Loss.rsg`.

    Parameters
    ----------
    stk_object : ansys.stk.core.stkobjects.Access
        The STK Access object.
    start_time : typing.Any
        The start time of the calculation (the default is None, which implies using the scenario start time).
    stop_time : typing.Any
        The stop time of the calculation (the default is None, which implies using the scenario stop time).
    step_time : float
        The step time for the calculation (the default is 60 seconds).
    colormap : matplotlib.colors.Colormap
        The colormap with which to color the data (the default is None).
    time_unit_abbreviation : str
        The time unit for formatting (the default is "UTCG").
    formatter : collections.abc.Callable[[float, float], str]
        The formatter for time axes (the default is None).

    Returns
    -------
    matplotlib.figure.Figure
        The newly created figure.
    matplotlib.axes.Axes
        The newly created axes.
    """
    root = stk_object.base.root
    start_time = start_time or root.current_scenario.start_time
    stop_time = stop_time or root.current_scenario.stop_time
    data = _get_access_data(stk_object, 'Radar Environment', False, None, ['Time', 'Rcvr Prop Atten', 'Xmtr Prop Atten'], start_time, stop_time, step)
    axes = [{'use_unit' : True, 'unit_squared': None, 'ylog10': False, 'y2log10': False, 'label': 'Ratio', 'lines': [
            {'y_name':'rcvr prop atten', 'label':'Rcvr Prop Atten', 'use_unit':True, 'unit_squared': None, 'dimension': 'Ratio'},
            {'y_name':'xmtr prop atten', 'label':'Xmtr Prop Atten', 'use_unit':True, 'unit_squared': None, 'dimension': 'Ratio'}]}]
    return line_chart(data, root, ['rcvr prop atten','xmtr prop atten'], ['time'], axes, 'time', 'Time', 'Radar Propagation Loss', colormap=colormap,  time_unit_abbreviation= time_unit_abbreviation, formatter= formatter)

def radar_sar_azimuth_resolution_line_chart(stk_object :Access, start_time : typing.Any = None, stop_time :typing.Any = None, step : float = 60, colormap: matplotlib.colors.Colormap = None,  time_unit_abbreviation: str = 'UTCG', formatter: collections.abc.Callable[[float, float], str] = None) -> tuple[matplotlib.figure.Figure, matplotlib.axes.Axes]:
    r"""Graph the radar SAR azimuth resolution and SAR integration time versus time.

    This graph wrapper was generated from `AGI\\STK12\\STKData\\Styles\\Access\\Radar SAR Azimuth Resolution.rsg`.

    Parameters
    ----------
    stk_object : ansys.stk.core.stkobjects.Access
        The STK Access object.
    start_time : typing.Any
        The start time of the calculation (the default is None, which implies using the scenario start time).
    stop_time : typing.Any
        The stop time of the calculation (the default is None, which implies using the scenario stop time).
    step_time : float
        The step time for the calculation (the default is 60 seconds).
    colormap : matplotlib.colors.Colormap
        The colormap with which to color the data (the default is None).
    time_unit_abbreviation : str
        The time unit for formatting (the default is "UTCG").
    formatter : collections.abc.Callable[[float, float], str]
        The formatter for time axes (the default is None).

    Returns
    -------
    matplotlib.figure.Figure
        The newly created figure.
    matplotlib.axes.Axes
        The newly created axes.
    """
    root = stk_object.base.root
    start_time = start_time or root.current_scenario.start_time
    stop_time = stop_time or root.current_scenario.stop_time
    data = _get_access_data(stk_object, 'Radar SAR', False, None, ['Time', 'SAR Integration Time', 'SAR Az Resolution'], start_time, stop_time, step)
    axes = [{'use_unit' : True, 'unit_squared': None, 'ylog10': False, 'y2log10': False, 'label': 'Distance', 'lines': [
            {'y_name':'sar az resolution', 'label':'SAR Az Resolution', 'use_unit':True, 'unit_squared': None, 'dimension': 'Distance'}]},
            {'use_unit' : True, 'unit_squared': None, 'ylog10': False, 'y2log10': False, 'label': 'Time', 'lines': [
            {'y_name':'sar integration time', 'label':'SAR Integration Time', 'use_unit':True, 'unit_squared': None, 'dimension': 'Time'}]}]
    return line_chart(data, root, ['sar az resolution','sar integration time'], ['time'], axes, 'time', 'Time', 'Radar SAR Azimuth Resolution', colormap=colormap,  time_unit_abbreviation= time_unit_abbreviation, formatter= formatter)

def radar_sar_time_resolution_line_chart(stk_object :Access, start_time : typing.Any = None, stop_time :typing.Any = None, step : float = 60, colormap: matplotlib.colors.Colormap = None,  time_unit_abbreviation: str = 'UTCG', formatter: collections.abc.Callable[[float, float], str] = None) -> tuple[matplotlib.figure.Figure, matplotlib.axes.Axes]:
    r"""Graph the time-varying data for the SAR time-resolution product.

    This graph wrapper was generated from `AGI\\STK12\\STKData\\Styles\\Access\\Radar SAR Time-Resolution.rsg`.

    Parameters
    ----------
    stk_object : ansys.stk.core.stkobjects.Access
        The STK Access object.
    start_time : typing.Any
        The start time of the calculation (the default is None, which implies using the scenario start time).
    stop_time : typing.Any
        The stop time of the calculation (the default is None, which implies using the scenario stop time).
    step_time : float
        The step time for the calculation (the default is 60 seconds).
    colormap : matplotlib.colors.Colormap
        The colormap with which to color the data (the default is None).
    time_unit_abbreviation : str
        The time unit for formatting (the default is "UTCG").
    formatter : collections.abc.Callable[[float, float], str]
        The formatter for time axes (the default is None).

    Returns
    -------
    matplotlib.figure.Figure
        The newly created figure.
    matplotlib.axes.Axes
        The newly created axes.
    """
    root = stk_object.base.root
    start_time = start_time or root.current_scenario.start_time
    stop_time = stop_time or root.current_scenario.stop_time
    data = _get_access_data(stk_object, 'Radar SAR', False, None, ['Time', 'SAR Time-Resolution Prod'], start_time, stop_time, step)
    axes = [{'use_unit' : True, 'unit_squared': None, 'ylog10': False, 'y2log10': False, 'label': 'Sar Time Res Prod', 'lines': [
            {'y_name':'sar time-resolution prod', 'label':'SAR Time-Resolution Prod', 'use_unit':True, 'unit_squared': None, 'dimension': 'SarTimeResProd'}]}]
    return line_chart(data, root, ['sar time-resolution prod'], ['time'], axes, 'time', 'Time', 'Radar SAR Time-Resolution', colormap=colormap,  time_unit_abbreviation= time_unit_abbreviation, formatter= formatter)

def radar_searchtrack_integration_line_chart(stk_object :Access, start_time : typing.Any = None, stop_time :typing.Any = None, step : float = 60, colormap: matplotlib.colors.Colormap = None,  time_unit_abbreviation: str = 'UTCG', formatter: collections.abc.Callable[[float, float], str] = None) -> tuple[matplotlib.figure.Figure, matplotlib.axes.Axes]:
    r"""Graph time-varying data for the following radar SearchTrack parameters: S/T integration time, S/T dwell time, and S/T pulses integrated.

    This graph wrapper was generated from `AGI\\STK12\\STKData\\Styles\\Access\\Radar SearchTrack Integration.rsg`.

    Parameters
    ----------
    stk_object : ansys.stk.core.stkobjects.Access
        The STK Access object.
    start_time : typing.Any
        The start time of the calculation (the default is None, which implies using the scenario start time).
    stop_time : typing.Any
        The stop time of the calculation (the default is None, which implies using the scenario stop time).
    step_time : float
        The step time for the calculation (the default is 60 seconds).
    colormap : matplotlib.colors.Colormap
        The colormap with which to color the data (the default is None).
    time_unit_abbreviation : str
        The time unit for formatting (the default is "UTCG").
    formatter : collections.abc.Callable[[float, float], str]
        The formatter for time axes (the default is None).

    Returns
    -------
    matplotlib.figure.Figure
        The newly created figure.
    matplotlib.axes.Axes
        The newly created axes.
    """
    root = stk_object.base.root
    start_time = start_time or root.current_scenario.start_time
    stop_time = stop_time or root.current_scenario.stop_time
    data = _get_access_data(stk_object, 'Radar SearchTrack', False, None, ['Time', 'S/T Dwell Time', 'S/T Integration Time', 'S/T Pulses Integrated'], start_time, stop_time, step)
    axes = [{'use_unit' : True, 'unit_squared': None, 'ylog10': False, 'y2log10': False, 'label': ' Time', 'lines': [
            {'y_name':'s/t integration time', 'label':'S/T Integration Time', 'use_unit':True, 'unit_squared': None, 'dimension': 'SmallTime'},
            {'y_name':'s/t dwell time', 'label':'S/T Dwell Time', 'use_unit':True, 'unit_squared': None, 'dimension': 'SmallTime'}]},
            {'use_unit' : False, 'unit_squared': None, 'ylog10': False, 'y2log10': False, 'label': 'Value', 'lines': [
            {'y_name':'s/t pulses integrated', 'label':'S/T Pulses Integrated', 'use_unit':False, 'unit_squared': None, 'dimension': 'None'}]}]
    return line_chart(data, root, ['s/t integration time','s/t dwell time','s/t pulses integrated'], ['time'], axes, 'time', 'Time', 'Radar SearchTrack Integration', colormap=colormap,  time_unit_abbreviation= time_unit_abbreviation, formatter= formatter)

def radar_searchtrack_snr_line_chart(stk_object :Access, start_time : typing.Any = None, stop_time :typing.Any = None, step : float = 60, colormap: matplotlib.colors.Colormap = None,  time_unit_abbreviation: str = 'UTCG', formatter: collections.abc.Callable[[float, float], str] = None) -> tuple[matplotlib.figure.Figure, matplotlib.axes.Axes]:
    r"""Graph radar SearchTrack signal-to-noise ratio versus time.

    This graph wrapper was generated from `AGI\\STK12\\STKData\\Styles\\Access\\Radar SearchTrack SNR.rsg`.

    Parameters
    ----------
    stk_object : ansys.stk.core.stkobjects.Access
        The STK Access object.
    start_time : typing.Any
        The start time of the calculation (the default is None, which implies using the scenario start time).
    stop_time : typing.Any
        The stop time of the calculation (the default is None, which implies using the scenario stop time).
    step_time : float
        The step time for the calculation (the default is 60 seconds).
    colormap : matplotlib.colors.Colormap
        The colormap with which to color the data (the default is None).
    time_unit_abbreviation : str
        The time unit for formatting (the default is "UTCG").
    formatter : collections.abc.Callable[[float, float], str]
        The formatter for time axes (the default is None).

    Returns
    -------
    matplotlib.figure.Figure
        The newly created figure.
    matplotlib.axes.Axes
        The newly created axes.
    """
    root = stk_object.base.root
    start_time = start_time or root.current_scenario.start_time
    stop_time = stop_time or root.current_scenario.stop_time
    data = _get_access_data(stk_object, 'Radar SearchTrack', False, None, ['Time', 'S/T SNR1'], start_time, stop_time, step)
    axes = [{'use_unit' : True, 'unit_squared': None, 'ylog10': False, 'y2log10': False, 'label': 'Ratio', 'lines': [
            {'y_name':'s/t snr1', 'label':'S/T SNR1', 'use_unit':True, 'unit_squared': None, 'dimension': 'Ratio'}]}]
    return line_chart(data, root, ['s/t snr1'], ['time'], axes, 'time', 'Time', 'Radar SearchTrack SNR', colormap=colormap,  time_unit_abbreviation= time_unit_abbreviation, formatter= formatter)

def radar_system_noise_line_chart(stk_object :Access, start_time : typing.Any = None, stop_time :typing.Any = None, step : float = 60, colormap: matplotlib.colors.Colormap = None,  time_unit_abbreviation: str = 'UTCG', formatter: collections.abc.Callable[[float, float], str] = None) -> tuple[matplotlib.figure.Figure, matplotlib.axes.Axes]:
    r"""Graph the antenna noise temperature and total noise temperature versus time for a radar receiver.

    This graph wrapper was generated from `AGI\\STK12\\STKData\\Styles\\Access\\Radar System Noise.rsg`.

    Parameters
    ----------
    stk_object : ansys.stk.core.stkobjects.Access
        The STK Access object.
    start_time : typing.Any
        The start time of the calculation (the default is None, which implies using the scenario start time).
    stop_time : typing.Any
        The stop time of the calculation (the default is None, which implies using the scenario stop time).
    step_time : float
        The step time for the calculation (the default is 60 seconds).
    colormap : matplotlib.colors.Colormap
        The colormap with which to color the data (the default is None).
    time_unit_abbreviation : str
        The time unit for formatting (the default is "UTCG").
    formatter : collections.abc.Callable[[float, float], str]
        The formatter for time axes (the default is None).

    Returns
    -------
    matplotlib.figure.Figure
        The newly created figure.
    matplotlib.axes.Axes
        The newly created axes.
    """
    root = stk_object.base.root
    start_time = start_time or root.current_scenario.start_time
    stop_time = stop_time or root.current_scenario.stop_time
    data = _get_access_data(stk_object, 'Radar Environment', False, None, ['System Temp', 'Time', 'Antenna Temp'], start_time, stop_time, step)
    axes = [{'use_unit' : True, 'unit_squared': None, 'ylog10': False, 'y2log10': False, 'label': 'Temperature', 'lines': [
            {'y_name':'antenna temp', 'label':'Antenna Temp', 'use_unit':True, 'unit_squared': None, 'dimension': 'Temperature'},
            {'y_name':'system temp', 'label':'System Temp', 'use_unit':True, 'unit_squared': None, 'dimension': 'Temperature'}]}]
    return line_chart(data, root, ['antenna temp','system temp'], ['time'], axes, 'time', 'Time', 'Radar System Noise', colormap=colormap,  time_unit_abbreviation= time_unit_abbreviation, formatter= formatter)

def revisit_diagram_interval_pie_chart(stk_object :Access, start_time: typing.Any = None, stop_time : typing.Any = None, color_list: list[typing.Any] = None) -> tuple[matplotlib.figure.Figure, matplotlib.axes.Axes]:
    r"""Create a pie chart showing the durations of access intervals and access gap intervals.

    This graph wrapper was generated from `AGI\\STK12\\STKData\\Styles\\Access\\Revisit Diagram.rsg`.

    Parameters
    ----------
    stk_object : ansys.stk.core.stkobjects.Access
        The STK Access object.
    start_time : typing.Any
        The start time of the calculation (the default is None, which implies using the scenario start time).
    stop_time : typing.Any
        The stop time of the calculation (the default is None, which implies using the scenario stop time).
    color_list : list of typing.Any
        The colors with which to color the pie chart slices (the default is None). Must have length >= 2.

    Returns
    -------
    matplotlib.figure.Figure
        The newly created figure.
    matplotlib.axes.Axes
        The newly created axes.
    """
    root = stk_object.base.root
    start_time = start_time or root.current_scenario.start_time
    stop_time = stop_time or root.current_scenario.stop_time
    df = stk_object.data_providers.item('Access Data').execute_elements(start_time, stop_time, ['Access Number', 'Stop Time', 'Start Time', 'Duration']).data_sets.to_pandas_dataframe()
    return interval_pie_chart(root, df, ['duration'], ['start time','stop time'], 'start time', 'stop time', start_time, stop_time, 'Revisit Diagram', 'Time', color_list = color_list)

def signal_to_noise_ratio_line_chart(stk_object :Access, start_time : typing.Any = None, stop_time :typing.Any = None, step : float = 60, colormap: matplotlib.colors.Colormap = None,  time_unit_abbreviation: str = 'UTCG', formatter: collections.abc.Callable[[float, float], str] = None) -> tuple[matplotlib.figure.Figure, matplotlib.axes.Axes]:
    r"""Plot the search track signal to noise ratio (S/N) over time, during each access interval.

    This graph wrapper was generated from `AGI\\STK12\\STKData\\Styles\\Access\\Signal_to_Noise_Ratio.rsg`.

    Parameters
    ----------
    stk_object : ansys.stk.core.stkobjects.Access
        The STK Access object.
    start_time : typing.Any
        The start time of the calculation (the default is None, which implies using the scenario start time).
    stop_time : typing.Any
        The stop time of the calculation (the default is None, which implies using the scenario stop time).
    step_time : float
        The step time for the calculation (the default is 60 seconds).
    colormap : matplotlib.colors.Colormap
        The colormap with which to color the data (the default is None).
    time_unit_abbreviation : str
        The time unit for formatting (the default is "UTCG").
    formatter : collections.abc.Callable[[float, float], str]
        The formatter for time axes (the default is None).

    Returns
    -------
    matplotlib.figure.Figure
        The newly created figure.
    matplotlib.axes.Axes
        The newly created axes.
    """
    root = stk_object.base.root
    start_time = start_time or root.current_scenario.start_time
    stop_time = stop_time or root.current_scenario.stop_time
    data = _get_access_data(stk_object, 'Radar SearchTrack', False, None, ['Time', 'S/T SNR1'], start_time, stop_time, step)
    axes = [{'use_unit' : True, 'unit_squared': None, 'ylog10': False, 'y2log10': False, 'label': 'Ratio', 'lines': [
            {'y_name':'s/t snr1', 'label':'S/T SNR1', 'use_unit':True, 'unit_squared': None, 'dimension': 'Ratio'}]}]
    return line_chart(data, root, ['s/t snr1'], ['time'], axes, 'time', 'Time', 'Signal to Noise Ratio', colormap=colormap,  time_unit_abbreviation= time_unit_abbreviation, formatter= formatter)

def sun_rfi_line_chart(stk_object :Access, start_time : typing.Any = None, stop_time :typing.Any = None, step : float = 60, colormap: matplotlib.colors.Colormap = None,  time_unit_abbreviation: str = 'UTCG', formatter: collections.abc.Callable[[float, float], str] = None) -> tuple[matplotlib.figure.Figure, matplotlib.axes.Axes]:
    r"""Graph the sun-induced antenna noise temperature as well as the receiver gain to system temperature ratio at the receiver as a function of time.

    This graph wrapper was generated from `AGI\\STK12\\STKData\\Styles\\Access\\Sun RFI.rsg`.

    Parameters
    ----------
    stk_object : ansys.stk.core.stkobjects.Access
        The STK Access object.
    start_time : typing.Any
        The start time of the calculation (the default is None, which implies using the scenario start time).
    stop_time : typing.Any
        The stop time of the calculation (the default is None, which implies using the scenario stop time).
    step_time : float
        The step time for the calculation (the default is 60 seconds).
    colormap : matplotlib.colors.Colormap
        The colormap with which to color the data (the default is None).
    time_unit_abbreviation : str
        The time unit for formatting (the default is "UTCG").
    formatter : collections.abc.Callable[[float, float], str]
        The formatter for time axes (the default is None).

    Returns
    -------
    matplotlib.figure.Figure
        The newly created figure.
    matplotlib.axes.Axes
        The newly created axes.
    """
    root = stk_object.base.root
    start_time = start_time or root.current_scenario.start_time
    stop_time = stop_time or root.current_scenario.stop_time
    data = _get_access_data(stk_object, 'Link Information', False, None, ['Time', 'Tsun', 'g/T'], start_time, stop_time, step)
    axes = [{'use_unit' : True, 'unit_squared': None, 'ylog10': False, 'y2log10': False, 'label': 'Temperature', 'lines': [
            {'y_name':'tsun', 'label':'Tsun', 'use_unit':True, 'unit_squared': None, 'dimension': 'Temperature'}]},
            {'use_unit' : False, 'unit_squared': None, 'ylog10': False, 'y2log10': False, 'label': 'Value', 'lines': [
            {'y_name':'g/t', 'label':'g/T (dB/K)', 'use_unit':False, 'unit_squared': None, 'dimension': 'None'}]}]
    return line_chart(data, root, ['tsun','g/t'], ['time'], axes, 'time', 'Time', 'Sun RFI', colormap=colormap,  time_unit_abbreviation= time_unit_abbreviation, formatter= formatter)




# Copyright (C) 2022 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Specify API mappings from old names to new names."""

import json
import logging
from pathlib import Path


class Mappings:
    """Mappings read from the XML mapping files."""

    def __init__(
        self,
        member_mappings,
        interface_mappings,
        class_mappings,
        enum_type_mappings,
        enum_value_mappings,
        namespace_mappings,
        named_argument_mappings,
    ):
        """Construct a new mapping based on the list of mapping for each category."""
        self.member_mappings = member_mappings
        self.interface_mappings = interface_mappings
        self.class_mappings = class_mappings
        self.enum_type_mappings = enum_type_mappings
        self.enum_value_mappings = enum_value_mappings
        self.namespace_mappings = namespace_mappings
        self.named_argument_mappings = named_argument_mappings

        self.old_member_names_of_interest = set([entry["old_member_name"] for entry in self.member_mappings])

        for argument_mapping in self.named_argument_mappings:
            member_name = argument_mapping["old_type_name"].split(".")[-1]
            self.old_member_names_of_interest.add(member_name)

        all_types = self.interface_mappings + self.class_mappings + self.enum_type_mappings

        self.old_type_names = set([entry["old_type_name"] for entry in all_types])

    @staticmethod
    def load_from(mappings_directory):
        """Load mappings from the specified directory."""
        json_files = Path(mappings_directory).glob("*.json")

        member_mappings = []
        interface_mappings = []
        class_mappings = []
        enum_type_mappings = []
        enum_value_mappings = []
        namespace_mappings = {}
        named_argument_mappings = []

        for json_file in json_files:
            with Path(json_file).open(mode="r") as f:
                mappings = json.load(f)

            logging.debug(f"Processing {json_file}")

            if "RootMapping" in mappings and "OldRootScope" in mappings["RootMapping"] and "NewRootScope" in mappings["RootMapping"]:
                old_root = mappings["RootMapping"]["OldRootScope"]
                new_root = mappings["RootMapping"]["NewRootScope"]
                namespace_mappings[old_root] = new_root
            else:
                old_root = ""
                new_root = ""

            method_entries = mappings.get("MemberMappings", [])

            for method_entry in method_entries:
                old_type_name = method_entry.get("ParentScope")
                old_member_name = method_entry.get("OldName")
                new_member_name = method_entry.get("NewName")

                member_mappings.append(
                    {
                        "old_root": old_root,
                        "old_type_name": old_type_name,
                        "old_member_name": old_member_name,
                        "new_member_name": new_member_name,
                    }
                )

            named_argument_entries = mappings.get("NamedArgumentMappings", [])

            for named_argument_entry in named_argument_entries:
                old_argument_name = named_argument_entry.get("OldName")
                new_argument_name = named_argument_entry.get("NewName")
                old_type_name = named_argument_entry.get("ParentScope")

                named_argument_mappings.append(
                    {
                        "old_root": old_root,
                        "old_type_name": old_type_name,
                        "old_argument_name": old_argument_name,
                        "new_argument_name": new_argument_name,
                    }
                )

            enum_type_entries = mappings.get("EnumTypeMappings", [])

            for enum_type_entry in enum_type_entries:
                old_enum_type_name = enum_type_entry.get("OldName")
                new_enum_type_name = enum_type_entry.get("NewName")

                enum_type_mappings.append(
                    {
                        "old_root": old_root,
                        "old_type_name": old_enum_type_name,
                        "new_type_name": new_enum_type_name,
                    }
                )

            enum_value_entries = mappings.get("EnumValueMappings", [])

            for enum_value_entry in enum_value_entries:
                old_enum_type_name = enum_value_entry.get("ParentScope")
                old_enum_value_name = enum_value_entry.get("OldName")
                new_enum_value_name = enum_value_entry.get("NewName")

                enum_value_mappings.append(
                    {
                        "old_root": old_root,
                        "old_type_name": old_enum_type_name,
                        "old_enum_value_name": old_enum_value_name,
                        "new_enum_value_name": new_enum_value_name,
                    }
                )

            interface_entries = mappings.get("InterfaceMappings", [])

            for interface_entry in interface_entries:
                old_interface_name = interface_entry.get("OldName")
                new_interface_type_name = interface_entry.get("NewName")

                interface_mappings.append(
                    {
                        "old_root": old_root,
                        "old_type_name": old_interface_name,
                        "new_type_name": new_interface_type_name,
                    }
                )

            class_entries = mappings.get("ClassMappings", [])

            for class_entry in class_entries:
                old_class_name = class_entry.get("OldName")
                new_class_name = class_entry.get("NewName")

                class_mappings.append(
                    {
                        "old_root": old_root,
                        "old_type_name": old_class_name,
                        "new_type_name": new_class_name,
                    }
                )

        return Mappings(
            member_mappings,
            interface_mappings,
            class_mappings,
            enum_type_mappings,
            enum_value_mappings,
            namespace_mappings,
            named_argument_mappings,
        )

    def is_member_name_of_interest(self, member_name):
        """Indicate if the specified member name could be of interest or not."""
        return member_name in self.old_member_names_of_interest or member_name == "__init__"

    def has_mapping_for_type(self, type_name):
        """Is there a mapping for the specified type?"""
        return type_name in self.old_type_names

    def get_mapping_for_named_argument(self, qualified_member_name, named_argument):
        """Get the mapping for the specified named argument."""
        matches = [
            entry
            for entry in self.named_argument_mappings
            if entry["old_type_name"] == qualified_member_name and entry["old_argument_name"] == named_argument
        ]
        if len(matches) == 1:
            return matches[0]["new_argument_name"]

        return None

    def get_replacement_for_member(self, type_name, member_name):
        """
        Get the new name for the specified type and member.

        If there is no mapping for this type and member, return the same name.
        """
        matches = [
            entry
            for entry in self.member_mappings
            if entry["old_type_name"] == type_name and entry["old_member_name"] == member_name
        ]
        if len(matches) == 1:
            return matches[0]["new_member_name"]
        elif len(matches) > 1:
            raise RuntimeError(f"Multiple mappings found for {type_name}.{member_name}")
        else:
            return member_name  # not renamed

    def get_replacement_for_enum(self, old_enum_type_name, old_enum_value_name):
        """
        Get the new name for the specified enumeration type and value.

        If there is no mapping for the specified enumeration type and value, return the same names.
        """
        enum_type_matches = [entry for entry in self.enum_type_mappings if entry["old_type_name"] == old_enum_type_name]
        new_enum_type_name = (
            enum_type_matches[0]["new_type_name"]
            if len(enum_type_matches) > 0  # pick the first one
            else old_enum_type_name
        )
        enum_value_matches = [
            entry
            for entry in self.enum_value_mappings
            if entry["old_type_name"] == old_enum_type_name and entry["old_enum_value_name"] == old_enum_value_name
        ]
        new_enum_value_name = (
            enum_value_matches[0]["new_enum_value_name"]
            if len(enum_value_matches) > 0  # pick the first one
            else old_enum_value_name
        )
        return (new_enum_type_name, new_enum_value_name)

    def get_replacement_for_type(self, root, type_name):
        """
        Get the new name for the specified type.

        If there is no mapping for the specified type, return the same name.
        """
        all_types = self.interface_mappings + self.class_mappings + self.enum_type_mappings
        matches = [
            entry
            for entry in all_types
            if entry["old_type_name"] == type_name and (root is None or len(root) == 0 or entry["old_root"] == root)
        ]
        if len(matches) == 1:
            return matches[0]["new_type_name"]
        elif len(matches) > 1:
            raise RuntimeError(f"Multiple mappings found for {root}.{type_name}")
        else:
            return type_name  # not renamed

    def get_replacement_for_namespace(self, namespace):
        """Get the new name for the specified namespace."""
        return self.namespace_mappings.get(namespace, namespace)
